use poise::serenity_prelude::Http;
use rocket::{http::CookieJar, State};
use rocket::response::stream::{Event, EventStream};

use crate::{cookies::user::USER_ID_COOKIE, models::Settings};

// TODO: Incrementally send guilds
#[get("/get_guilds")]
pub async fn get_guilds<'a>(
    cookies: &'a CookieJar<'_>,
    settings: &'a State<Settings>,
    http: &'a State<Http>,
) -> EventStream![Event + 'a] {
    // EventStream![] is shorthand for EventStream[Event],
    // but we need to pass in a lifetime parameter.
    EventStream! {
        // TODO: Proper error handling
        let user_id: u64 = match cookies.get_private(USER_ID_COOKIE) {
            Some(id) => match id.value().parse() {
                Ok(id) => id,
                Err(_) => panic!("BadRequest"),
                // Err(_) => return Err(Status::BadRequest),
            },
            None => panic!("Unauthorized"),
            // None => return Err(Status::Unauthorized),
        };
        for guild in &settings.guilds {
            if guild.hidden {
                continue;
            }
            yield Event::data(format!("{},{}", guild.id, http.get_member(guild.id, user_id).await.is_ok()));
        }
    }
}
