use chrono::{Duration, Utc};
use rocket::{
    form::Form,
    http::{Cookie, CookieJar},
    response::Redirect,
};

use crate::{cookies::{token::TOKEN_EXPIRE_COOKIE, WELCOMED_COOKIE}, models::SessionUser};

#[derive(FromForm)]
pub struct Login<'r> {
    token_type: &'r str,
    access_token: &'r str,
    expires_in: u64,
    scope: &'r str,
}

#[post("/login", data = "<login>")]
pub async fn post_login(login: Form<Login<'_>>, cookies: &CookieJar<'_>) -> Redirect {
    if (login.token_type != "Bearer" || login.scope.split('+').any(|scope| scope == "identify"))
        && SessionUser::init(login.access_token, cookies).await.is_ok()
    {
        cookies.add(Cookie::new(
            TOKEN_EXPIRE_COOKIE,
            (Utc::now() + Duration::seconds(login.expires_in as i64))
                .timestamp()
                .to_string(),
        ));
        cookies.remove(Cookie::named(WELCOMED_COOKIE));
    }
    Redirect::to("/")
}
